==> auroraview-0.2.4.tar.gz: auroraview-0.2.4/src/ipc/json_bindings.rs <==
//! Python bindings for high-performance JSON operations
//!
//! This module exposes our SIMD-accelerated JSON functions to Python,
//! providing orjson-equivalent performance without requiring any Python dependencies.
//!
//! ## Usage in Python:
//! ```python
//! from auroraview._auroraview import json_loads, json_dumps
//!
//! # Parse JSON (2-3x faster than json.loads)
//! data = json_loads('{"key": "value"}')
//!
//! # Serialize JSON (2-3x faster than json.dumps)
//! json_str = json_dumps({"key": "value"})
//! ```

use super::json;
use pyo3::prelude::*;
use pyo3::types::PyBytes;

/// Parse JSON string to Python object (SIMD-accelerated)
///
/// This is equivalent to orjson.loads() but without requiring orjson as a dependency.
/// Uses simd-json for 2-3x faster parsing than standard json.loads().
///
/// # Arguments
/// * `data` - JSON string or bytes to parse
///
/// # Returns
/// Python object (dict, list, str, int, float, bool, or None)
///
/// # Example
/// ```python
/// data = json_loads('{"name": "test", "value": 123}')
/// print(data)  # {'name': 'test', 'value': 123}
/// ```
#[pyfunction]
pub fn json_loads(py: Python, data: &Bound<'_, PyAny>) -> PyResult<PyObject> {
    // Accept both str and bytes
    let json_str = if let Ok(s) = data.extract::<String>() {
        s
    } else if let Ok(b) = data.extract::<&[u8]>() {
        String::from_utf8(b.to_vec()).map_err(|e| {
            PyErr::new::<pyo3::exceptions::PyValueError, _>(format!("Invalid UTF-8: {}", e))
        })?
    } else {
        return Err(PyErr::new::<pyo3::exceptions::PyTypeError, _>(
            "Expected str or bytes",
        ));
    };

    // Parse using SIMD-accelerated parser
    let value =
        json::from_str(&json_str).map_err(PyErr::new::<pyo3::exceptions::PyValueError, _>)?;

    // Convert to Python object
    json::json_to_python(py, &value)
}

/// Serialize Python object to JSON string (SIMD-accelerated)
///
/// This is equivalent to orjson.dumps() but without requiring orjson as a dependency.
/// Uses optimized serialization for 2-3x faster performance than json.dumps().
///
/// # Arguments
/// * `obj` - Python object to serialize (dict, list, str, int, float, bool, or None)
///
/// # Returns
/// JSON string
///
/// # Example
/// ```python
/// json_str = json_dumps({"name": "test", "value": 123})
/// print(json_str)  # '{"name":"test","value":123}'
/// ```
#[pyfunction]
pub fn json_dumps(_py: Python, obj: &Bound<'_, PyAny>) -> PyResult<String> {
    // Convert Python object to JSON Value
    let value = json::python_to_json(obj)?;

    // Serialize to string
    json::to_string(&value).map_err(PyErr::new::<pyo3::exceptions::PyValueError, _>)
}

/// Serialize Python object to JSON bytes (SIMD-accelerated, zero-copy)
///
/// This is the most efficient serialization method, returning bytes directly
/// without string conversion overhead. Equivalent to orjson.dumps() with bytes output.
///
/// # Arguments
/// * `obj` - Python object to serialize
///
/// # Returns
/// JSON bytes (can be used directly for network transmission)
///
/// # Example
/// ```python
/// json_bytes = json_dumps_bytes({"name": "test"})
/// print(json_bytes)  # b'{"name":"test"}'
/// ```
#[pyfunction]
pub fn json_dumps_bytes(py: Python, obj: &Bound<'_, PyAny>) -> PyResult<Py<PyBytes>> {
    // Convert Python object to JSON Value
    let value = json::python_to_json(obj)?;

    // Serialize to string
    let json_str =
        json::to_string(&value).map_err(PyErr::new::<pyo3::exceptions::PyValueError, _>)?;

    // Return as bytes (zero-copy)
    Ok(PyBytes::new(py, json_str.as_bytes()).into())
}

/// Register JSON functions with Python module
pub fn register_json_functions(m: &Bound<'_, PyModule>) -> PyResult<()> {
    m.add_function(wrap_pyfunction!(json_loads, m)?)?;
    m.add_function(wrap_pyfunction!(json_dumps, m)?)?;
    m.add_function(wrap_pyfunction!(json_dumps_bytes, m)?)?;
    Ok(())
}

#[cfg(test)]
mod tests {
    use super::*;
    use pyo3::types::{PyDict, PyString};
    use pyo3::Python;

    #[test]
    fn test_json_roundtrip() {
        Python::with_gil(|py| {
            // Create a Python dict
            let dict = PyDict::new(py);
            dict.set_item("name", "test").unwrap();
            dict.set_item("value", 123).unwrap();

            // Serialize
            let json_str = json_dumps(py, &dict).unwrap();
            assert!(json_str.contains("name"));
            assert!(json_str.contains("test"));

            // Parse back
            let json_str_bound = PyString::new(py, &json_str);
            let parsed = json_loads(py, json_str_bound.as_any()).unwrap();
            let parsed_dict = parsed.downcast_bound::<PyDict>(py).unwrap();

            assert_eq!(
                parsed_dict
                    .get_item("name")
                    .unwrap()
                    .unwrap()
                    .extract::<String>()
                    .unwrap(),
                "test"
            );
            assert_eq!(
                parsed_dict
                    .get_item("value")
                    .unwrap()
                    .unwrap()
                    .extract::<i64>()
                    .unwrap(),
                123
            );
        });
    }
}

==> auroraview-0.2.5.tar.gz: auroraview-0.2.5/src/ipc/json_bindings.rs <==
//! Python bindings for high-performance JSON operations
//!
//! This module exposes our SIMD-accelerated JSON functions to Python,
//! providing orjson-equivalent performance without requiring any Python dependencies.
//!
//! ## Usage in Python:
//! ```python
//! from auroraview._auroraview import json_loads, json_dumps
//!
//! # Parse JSON (2-3x faster than json.loads)
//! data = json_loads('{"key": "value"}')
//!
//! # Serialize JSON (2-3x faster than json.dumps)
//! json_str = json_dumps({"key": "value"})
//! ```

use super::json;
use pyo3::prelude::*;
use pyo3::types::PyBytes;
use pyo3::Py;

/// Parse JSON string to Python object (SIMD-accelerated)
///
/// This is equivalent to orjson.loads() but without requiring orjson as a dependency.
/// Uses simd-json for 2-3x faster parsing than standard json.loads().
///
/// # Arguments
/// * `data` - JSON string or bytes to parse
///
/// # Returns
/// Python object (dict, list, str, int, float, bool, or None)
///
/// # Example
/// ```python
/// data = json_loads('{"name": "test", "value": 123}')
/// print(data)  # {'name': 'test', 'value': 123}
/// ```
#[pyfunction]
pub fn json_loads(py: Python, data: &Bound<'_, PyAny>) -> PyResult<Py<PyAny>> {
    // Accept both str and bytes
    let json_str = if let Ok(s) = data.extract::<String>() {
        s
    } else if let Ok(b) = data.extract::<&[u8]>() {
        String::from_utf8(b.to_vec()).map_err(|e| {
            PyErr::new::<pyo3::exceptions::PyValueError, _>(format!("Invalid UTF-8: {}", e))
        })?
    } else {
        return Err(PyErr::new::<pyo3::exceptions::PyTypeError, _>(
            "Expected str or bytes",
        ));
    };

    // Parse using SIMD-accelerated parser
    let value =
        json::from_str(&json_str).map_err(PyErr::new::<pyo3::exceptions::PyValueError, _>)?;

    // Convert to Python object
    json::json_to_python(py, &value)
}

/// Serialize Python object to JSON string (SIMD-accelerated)
///
/// This is equivalent to orjson.dumps() but without requiring orjson as a dependency.
/// Uses optimized serialization for 2-3x faster performance than json.dumps().
///
/// # Arguments
/// * `obj` - Python object to serialize (dict, list, str, int, float, bool, or None)
///
/// # Returns
/// JSON string
///
/// # Example
/// ```python
/// json_str = json_dumps({"name": "test", "value": 123})
/// print(json_str)  # '{"name":"test","value":123}'
/// ```
#[pyfunction]
pub fn json_dumps(_py: Python, obj: &Bound<'_, PyAny>) -> PyResult<String> {
    // Convert Python object to JSON Value
    let value = json::python_to_json(obj)?;

    // Serialize to string
    json::to_string(&value).map_err(PyErr::new::<pyo3::exceptions::PyValueError, _>)
}

/// Serialize Python object to JSON bytes (SIMD-accelerated, zero-copy)
///
/// This is the most efficient serialization method, returning bytes directly
/// without string conversion overhead. Equivalent to orjson.dumps() with bytes output.
///
/// # Arguments
/// * `obj` - Python object to serialize
///
/// # Returns
/// JSON bytes (can be used directly for network transmission)
///
/// # Example
/// ```python
/// json_bytes = json_dumps_bytes({"name": "test"})
/// print(json_bytes)  # b'{"name":"test"}'
/// ```
#[pyfunction]
pub fn json_dumps_bytes(py: Python, obj: &Bound<'_, PyAny>) -> PyResult<Py<PyBytes>> {
    // Convert Python object to JSON Value
    let value = json::python_to_json(obj)?;

    // Serialize to string
    let json_str =
        json::to_string(&value).map_err(PyErr::new::<pyo3::exceptions::PyValueError, _>)?;

    // Return as bytes (zero-copy)
    Ok(PyBytes::new(py, json_str.as_bytes()).into())
}

/// Register JSON functions with Python module
pub fn register_json_functions(m: &Bound<'_, PyModule>) -> PyResult<()> {
    m.add_function(wrap_pyfunction!(json_loads, m)?)?;
    m.add_function(wrap_pyfunction!(json_dumps, m)?)?;
    m.add_function(wrap_pyfunction!(json_dumps_bytes, m)?)?;
    Ok(())
}

#[cfg(test)]
mod tests {
    use super::*;
    use pyo3::types::{PyDict, PyString};
    use pyo3::Python;

    #[test]
    fn test_json_roundtrip() {
        Python::attach(|py| {
            // Create a Python dict
            let dict = PyDict::new(py);
            dict.set_item("name", "test").unwrap();
            dict.set_item("value", 123).unwrap();

            // Serialize
            let json_str = json_dumps(py, &dict).unwrap();
            assert!(json_str.contains("name"));
            assert!(json_str.contains("test"));

            // Parse back
            let json_str_bound = PyString::new(py, &json_str);
            let parsed = json_loads(py, json_str_bound.as_any()).unwrap();
            let parsed_dict = parsed.bind(py).cast::<PyDict>().unwrap();

            assert_eq!(
                parsed_dict
                    .get_item("name")
                    .unwrap()
                    .unwrap()
                    .extract::<String>()
                    .unwrap(),
                "test"
            );
            assert_eq!(
                parsed_dict
                    .get_item("value")
                    .unwrap()
                    .unwrap()
                    .extract::<i64>()
                    .unwrap(),
                123
            );
            Ok::<(), pyo3::PyErr>(())
        })
        .unwrap();
    }
}
